@@ -1,7 +1,7 @@
 # -*- coding: UTF-8 -*-
 """Expand seeds into dependency-closed lists of packages."""
 
-# Copyright (c) 2004, 2005, 2006, 2007 Canonical Ltd.
+# Copyright (c) 2004, 2005, 2006, 2007, 2008 Canonical Ltd.
 #
 # Germinate is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -29,6 +29,9 @@ except NameError:
     import sets
     set = sets.Set
 
+import Germinate.seeds
+import Germinate.tsort
+
 class Germinator:
     PROGRESS = 15
 
@@ -39,7 +42,7 @@ class Germinator:
         self.sources = {}
         self.pruned = {}
 
-        self.structure = ''
+        self.structure = []
         self.seeds = []
         self.seed = {}
         self.seedrecommends = {}
@@ -48,6 +51,7 @@ class Germinator:
         self.substvars = {}
         self.depends = {}
         self.build_depends = {}
+        self.supported = None
 
         self.sourcepkgs = {}
         self.build_sourcepkgs = {}
@@ -90,7 +94,7 @@ class Germinator:
     def error(self, msg, *args, **kwargs):
         logging.error(msg, *args, **kwargs)
 
-    def parseStructure(self, f):
+    def parseStructureFile(self, f):
         """Parse a seed structure file. This is an ordered sequence of lines
         as follows:
 
@@ -105,24 +109,124 @@ class Germinator:
         The lines should be topologically sorted with respect to
         inheritance, with inherited-from seeds at the start.
 
-        Returns (ordered list of seed names, dict of SEED -> INHERITED)."""
-        lines = []
+        Any line as follows:
+
+        include BRANCH
+
+        causes another seed branch to be included. Seed names will be
+        resolved in included branches if they cannot be found in the current
+        branch.
+
+        Returns (ordered list of seed names, dict of SEED -> INHERITED,
+        branches, structure)."""
         seednames = []
         seedinherit = {}
+        seedbranches = []
+        lines = []
 
         for line in f:
-            lines.append(line)
+            line = line.strip()
+            if not line:
+                continue
+            if line.startswith('#'):
+                continue
             words = line.split()
             if words[0].endswith(':'):
                 seed = words[0][:-1]
                 seednames.append(seed)
                 seedinherit[seed] = list(words[1:])
+                lines.append(line)
+            elif words[0] == 'include':
+                seedbranches.extend(words[1:])
             else:
                 self.error("Unparseable seed structure entry: %s", line)
         f.close()
 
-        self.structure = ''.join(lines)
-        return (seednames, seedinherit)
+        return (seednames, seedinherit, seedbranches, lines)
+
+    def parseStructure(self, seed_base, branch, bzr=False, got_branches=None):
+        """Like parseStructureFile, but deals with acquiring the seed
+        structure files and recursively acquiring any seed structure files
+        it includes. got_branches is for internal use only."""
+        # TODO: this method's signature is awful; it returns different types
+        # depending on whether it is called recursively or at the top level.
+        if got_branches is None:
+            top_level = True
+            got_branches = set()
+        else:
+            top_level = False
+        all_names = []
+        all_inherit = {}
+        all_branches = []
+        all_structure = []
+
+        if branch in got_branches:
+            return all_names, all_inherit, all_branches, all_structure
+
+        # Fetch this one
+        seed = Germinate.seeds.open_seed(seed_base, branch, "STRUCTURE", bzr)
+        names, inherit, branches, structure = self.parseStructureFile(seed)
+        branches.insert(0, branch)
+        got_branches.add(branch)
+
+        # Recursively expand included branches
+        for child_branch in branches:
+            child_names, child_inherit, child_branches, child_structure = \
+                self.parseStructure(seed_base, child_branch, bzr, got_branches)
+            for grandchild_name in child_names:
+                all_names.append(grandchild_name)
+            all_inherit.update(child_inherit)
+            for grandchild_branch in child_branches:
+                if grandchild_branch not in all_branches:
+                    all_branches.append(grandchild_branch)
+            for child_structure_line in child_structure:
+                child_structure_name = child_structure_line.split()[0][:-1]
+                for i in range(len(all_structure)):
+                    if all_structure[i].split()[0][:-1] == child_structure_name:
+                        del all_structure[i]
+                        break
+                all_structure.append(child_structure_line)
+
+        # Attach the main branch's data to the end
+        for child_name in names:
+            all_names.append(child_name)
+        all_inherit.update(inherit)
+        for child_branch in branches:
+            if child_branch not in all_branches:
+                all_branches.append(child_branch)
+        for structure_line in structure:
+            structure_name = structure_line.split()[0][:-1]
+            for i in range(len(all_structure)):
+                if all_structure[i].split()[0][:-1] == structure_name:
+                    del all_structure[i]
+                    break
+            all_structure.append(structure_line)
+
+        # We generally want to process branches in reverse order, so that
+        # later branches can override seeds from earlier branches
+        all_branches.reverse()
+
+        # Expand out incomplete inheritance lists
+        order = Germinate.tsort.topo_sort(all_inherit)
+        for name in order:
+            seen = set()
+            new_inherit = []
+            for inheritee in all_inherit[name]:
+                for expanded in all_inherit[inheritee]:
+                    if expanded not in seen:
+                        new_inherit.append(expanded)
+                        seen.add(expanded)
+                if inheritee not in seen:
+                    new_inherit.append(inheritee)
+                    seen.add(inheritee)
+            all_inherit[name] = new_inherit
+
+        if top_level:
+            self.structure = all_structure
+            self.supported = all_names[-1]
+            return order, all_inherit, all_branches
+        else:
+            return all_names, all_inherit, all_branches, all_structure
 
     def parseHints(self, f):
         """Parse a hints file."""
@@ -136,66 +240,95 @@ class Germinator:
             self.hints[words[1]] = words[0]
         f.close()
 
-    def parsePackages(self, f, pkgtype):
+    def parsePackages(self, tag_files, pkgtype):
         """Parse a Packages file and get the information we need."""
-        p = apt_pkg.ParseTagFile(f)
-        while p.Step() == 1:
-            pkg = p.Section["Package"]
-            self.packages[pkg] = {}
-            self.packagetype[pkg] = pkgtype
-            self.pruned[pkg] = set()
+        for f in tag_files:
+            p = apt_pkg.ParseTagFile(f)
+            while p.Step() == 1:
+                pkg = p.Section["Package"]
+                ver = p.Section["Version"]
+                last_ver = None
 
-            self.packages[pkg]["Maintainer"] = \
-                unicode(p.Section.get("Maintainer", ""), "utf8", "replace")
+                # If there is a previous package info stored, fetch
+                # the version to compare them.
+                if self.packages.has_key(pkg):
+                    last_ver = self.packages[pkg]["Version"]
 
-            for field in "Pre-Depends", "Depends", "Recommends", "Suggests":
-                value = p.Section.get(field, "")
-                self.packages[pkg][field] = apt_pkg.ParseDepends(value)
+                # If this is a new package, or if the stored version
+                # is older than the new version, store the new
+                # package.
+                if (not self.packages.has_key(pkg) or
+                    apt_pkg.VersionCompare(last_ver, ver) < 0):
+                    self.packages[pkg] = {}
+                    self.packagetype[pkg] = pkgtype
+                    self.pruned[pkg] = set()
 
-            for field in "Size", "Installed-Size":
-                value = p.Section.get(field, "0")
-                self.packages[pkg][field] = int(value)
+                    self.packages[pkg]["Section"] = \
+                        p.Section.get("Section", "").split('/')[-1]
 
-            src = p.Section.get("Source", pkg)
-            idx = src.find("(")
-            if idx != -1:
-                src = src[:idx].strip()
-            self.packages[pkg]["Source"] = src
+                    self.packages[pkg]["Version"] = p.Section.get("Version")
 
-            provides = apt_pkg.ParseDepends(p.Section.get("Provides", ""))
-            for prov in provides:
-                if prov[0][0] not in self.provides:
-                    self.provides[prov[0][0]] = []
-                    if prov[0][0] in self.packages:
-                        self.provides[prov[0][0]].append(prov[0][0])
-                self.provides[prov[0][0]].append(pkg)
-            self.packages[pkg]["Provides"] = provides
+                    self.packages[pkg]["Maintainer"] = \
+                        unicode(p.Section.get("Maintainer", ""), "utf8", "replace")
 
-            if pkg in self.provides:
-                self.provides[pkg].append(pkg)
+                    for field in "Pre-Depends", "Depends", "Recommends", "Suggests":
+                        value = p.Section.get(field, "")
+                        self.packages[pkg][field] = apt_pkg.ParseDepends(value)
 
-            self.packages[pkg]["Kernel-Version"] = p.Section.get("Kernel-Version", "")
-        f.close()
+                    for field in "Size", "Installed-Size":
+                        value = p.Section.get(field, "0")
+                        self.packages[pkg][field] = int(value)
 
-    def parseSources(self, f):
+                    src = p.Section.get("Source", pkg)
+                    idx = src.find("(")
+                    if idx != -1:
+                        src = src[:idx].strip()
+                    self.packages[pkg]["Source"] = src
+
+                    provides = apt_pkg.ParseDepends(p.Section.get("Provides", ""))
+                    for prov in provides:
+                        if prov[0][0] not in self.provides:
+                            self.provides[prov[0][0]] = []
+                            if prov[0][0] in self.packages:
+                                self.provides[prov[0][0]].append(prov[0][0])
+                        self.provides[prov[0][0]].append(pkg)
+                    self.packages[pkg]["Provides"] = provides
+
+                    if pkg in self.provides:
+                        self.provides[pkg].append(pkg)
+
+                    self.packages[pkg]["Kernel-Version"] = p.Section.get("Kernel-Version", "")
+            f.close()
+
+    def parseSources(self, tag_files):
         """Parse a Sources file and get the information we need."""
-        p = apt_pkg.ParseTagFile(f)
-        while p.Step() == 1:
-            src = p.Section["Package"]
-            self.sources[src] = {}
+        for f in tag_files:
+            p = apt_pkg.ParseTagFile(f)
+            while p.Step() == 1:
+                src = p.Section["Package"]
+                ver = p.Section["Version"]
+                last_ver = None
 
-            self.sources[src]["Maintainer"] = \
-                unicode(p.Section.get("Maintainer", ""), "utf8", "replace")
-            self.sources[src]["IPv6"] = "Unknown"
+                if self.sources.has_key(src):
+                    last_ver = self.sources[src]["Version"]
 
-            for field in "Build-Depends", "Build-Depends-Indep":
-                value = p.Section.get(field, "")
-                self.sources[src][field] = apt_pkg.ParseSrcDepends(value)
+                if (not self.sources.has_key(src) or
+                    apt_pkg.VersionCompare(last_ver, ver) < 0):
+                    self.sources[src] = {}
 
-            binaries = apt_pkg.ParseDepends(p.Section.get("Binary", src))
-            self.sources[src]["Binaries"] = [ bin[0][0] for bin in binaries ]
+                    self.sources[src]["Maintainer"] = \
+                        unicode(p.Section.get("Maintainer", ""), "utf8", "replace")
+                    self.sources[src]["IPv6"] = "Unknown"
+                    self.sources[src]["Version"] = ver
 
-        f.close()
+                    for field in "Build-Depends", "Build-Depends-Indep":
+                        value = p.Section.get(field, "")
+                        self.sources[src][field] = apt_pkg.ParseSrcDepends(value)
+
+                    binaries = apt_pkg.ParseDepends(p.Section.get("Binary", src))
+                    self.sources[src]["Binaries"] = [ bin[0][0] for bin in binaries ]
+
+            f.close()
 
     def parseIPv6(self, f):
         """Parse the IPv6 dailydump file and get the information we need."""
@@ -517,7 +650,7 @@ class Germinator:
         """Weed out blacklisted seed entries from a list."""
         white = []
         if build_tree:
-            outerseeds = ["supported"]
+            outerseeds = [self.supported]
         else:
             outerseeds = self.outerSeeds(seedname)
         for pkg in pkgs:
@@ -559,7 +692,7 @@ class Germinator:
                     break
             self.rescueIncludes(seedname, "extra", build_tree=False)
 
-        self.rescueIncludes("supported", "extra", build_tree=True)
+        self.rescueIncludes(self.supported, "extra", build_tree=True)
 
     def addExtras(self, seedrelease=None):
         """Add packages generated by the sources but not in any seed."""
@@ -634,7 +767,10 @@ class Germinator:
     def reverseDepends(self):
         """Calculate the reverse dependency relationships."""
         for pkg in self.all:
-            for field in "Pre-Depends", "Depends":
+            fields = ["Pre-Depends", "Depends"]
+            if self.packages[pkg]["Section"] == "metapackages":
+                fields.append("Recommends")
+            for field in fields:
                 for deplist in self.packages[pkg][field]:
                     for dep in deplist:
                         if dep[0] in self.all and \
@@ -653,8 +789,11 @@ class Germinator:
             if "Reverse-Depends" not in self.packages[pkg]:
                 continue
 
-            for field in ("Pre-Depends", "Depends",
-                          "Build-Depends", "Build-Depends-Indep"):
+            fields = ["Pre-Depends", "Depends"]
+            if self.packages[pkg]["Section"] == "metapackages":
+                fields.append("Recommends")
+            fields.extend(["Build-Depends", "Build-Depends-Indep"])
+            for field in fields:
                 if field not in self.packages[pkg]["Reverse-Depends"]:
                     continue
 
@@ -820,7 +959,7 @@ class Germinator:
             self.warning("Pruned %s from %s", pkg, seedname)
             return
         if build_tree:
-            outerseeds = ["supported"]
+            outerseeds = [self.supported]
         else:
             outerseeds = self.outerSeeds(seedname)
         for outerseed in outerseeds:
@@ -870,6 +1009,12 @@ class Germinator:
                                second_class=second_class,
                                build_tree=build_tree)
 
+        if self.packages[pkg]["Section"] == "metapackages":
+            self.addDependencyTree(seedname, pkg,
+                                   self.packages[pkg]["Recommends"],
+                                   second_class=second_class,
+                                   build_tree=build_tree)
+
         src = self.packages[pkg]["Source"]
         if src not in self.sources:
             self.error("Missing source package: %s (for %s)", src, pkg)
@@ -911,7 +1056,9 @@ class Germinator:
         """Automatically rescue packages matching certain patterns from
         other seeds."""
 
-        if seedname not in self.seeds or rescue_seedname not in self.seeds:
+        if seedname not in self.seeds and seedname != "extra":
+            return
+        if rescue_seedname not in self.seeds and rescue_seedname != "extra":
             return
 
         # Find all the source packages.
