@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 """Update list files from the Wiki."""
 
-# Copyright (c) 2004, 2005, 2006, 2007 Canonical Ltd.
+# Copyright (c) 2004, 2005, 2006, 2007, 2008 Canonical Ltd.
 #
 # Germinate is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -44,15 +44,17 @@ import Germinate.version
 
 
 # Where do we get up-to-date seeds from?
-SEEDS = "http://people.ubuntu.com/~ubuntu-archive/seeds/"
-SEEDS_BZR = "http://bazaar.launchpad.net/~ubuntu-core-dev/ubuntu-seeds/"
-RELEASE = "ubuntu.gutsy"
+SEEDS = ["http://people.ubuntu.com/~ubuntu-archive/seeds/"]
+SEEDS_BZR = ["http://bazaar.launchpad.net/~ubuntu-core-dev/ubuntu-seeds/"]
+RELEASE = "ubuntu.hardy"
 
 # If we need to download Packages.gz and/or Sources.gz, where do we get
 # them from?
-MIRROR = "http://archive.ubuntu.com/ubuntu/"
-SOURCE_MIRROR = None
-DIST = ["gutsy"]
+MIRRORS = []
+SOURCE_MIRRORS = []
+DEFAULT_MIRROR = "http://archive.ubuntu.com/ubuntu/"
+DEFAULT_SOURCE_MIRROR = None
+DIST = ["hardy"]
 COMPONENTS = ["main", "restricted"]
 ARCH = "i386"
 
@@ -246,9 +248,15 @@ def write_prov_list(filename, provdict):
 
 def write_structure(filename, structure):
     f = open(filename, "w")
-    print >>f, structure,
+    for line in structure:
+        print >>f, line
     f.close()
 
+def write_seedtext(filename, seedtext):
+    f = open(filename, "w")
+    for line in seedtext:
+        print >>f, line.rstrip('\n')
+
 
 def usage(f):
     print >>f, """Usage: germinate.py [options]
@@ -278,11 +286,13 @@ Options:
   --seed-packages=PARENT/PKG,PARENT/PKG,...
                         Treat each PKG as a seed by itself, inheriting from
                         PARENT.
-""" % (SEEDS, RELEASE, MIRROR, ",".join(DIST), ARCH, ",".join(COMPONENTS))
+""" % (",".join(SEEDS), RELEASE, DEFAULT_MIRROR, ",".join(DIST), ARCH,
+       ",".join(COMPONENTS))
 
 
 def main():
-    global SEEDS, SEEDS_BZR, RELEASE, MIRROR, SOURCE_MIRROR
+    global SEEDS, SEEDS_BZR, RELEASE
+    global DEFAULT_MIRROR, DEFAULT_SOURCE_MIRROR, SOURCE_MIRRORS, MIRRORS
     global DIST, ARCH, COMPONENTS, CHECK_IPV6
     verbose = False
     bzr = False
@@ -325,20 +335,18 @@ def main():
         elif option in ("-v", "--verbose"):
             verbose = True
         elif option in ("-S", "--seed-source"):
-            SEEDS = value
-            if not SEEDS.endswith("/"):
-                SEEDS += "/"
+            SEEDS = value.split(",")
             seeds_set = True
         elif option in ("-s", "--seed-dist"):
             RELEASE = value
         elif option in ("-m", "--mirror"):
-            MIRROR = value
-            if not MIRROR.endswith("/"):
-                MIRROR += "/"
+            if not value.endswith("/"):
+                value += "/"
+            MIRRORS.append(value)
         elif option == "--source-mirror":
-            SOURCE_MIRROR = value
-            if not SOURCE_MIRROR.endswith("/"):
-                SOURCE_MIRROR += "/"
+            if not value.endswith("/"):
+                value += "/"
+            SOURCE_MIRRORS.append(value)
         elif option in ("-d", "--dist"):
             DIST = value.split(",")
         elif option in ("-c", "--components"):
@@ -358,6 +366,9 @@ def main():
         elif option == "--seed-packages":
             seed_packages = value.split(',')
 
+    if not MIRRORS:
+        MIRRORS.append(DEFAULT_MIRROR)
+
     logger = logging.getLogger()
     if verbose:
         logger.setLevel(logging.DEBUG)
@@ -369,8 +380,9 @@ def main():
 
     apt_pkg.InitConfig()
     apt_pkg.Config.Set("APT::Architecture", ARCH)
+    apt_pkg.InitSystem()
 
-    Germinate.Archive.TagFile(MIRROR, SOURCE_MIRROR).feed(
+    Germinate.Archive.TagFile(MIRRORS, SOURCE_MIRRORS).feed(
         g, DIST, COMPONENTS, ARCH, cleanup)
 
     if CHECK_IPV6:
@@ -379,14 +391,18 @@ def main():
     if os.path.isfile("hints"):
         g.parseHints(open("hints"))
 
-    blacklist = Germinate.seeds.open_seed(SEEDS + RELEASE, "blacklist", bzr)
+    blacklist = Germinate.seeds.open_seed(SEEDS, RELEASE, "blacklist", bzr)
     if blacklist is not None:
         g.parseBlacklist(blacklist)
 
-    (seednames, seedinherit) = g.parseStructure(
-        Germinate.seeds.open_seed(SEEDS + RELEASE, "STRUCTURE", bzr))
+    seednames, seedinherit, seedbranches = g.parseStructure(
+        SEEDS, RELEASE, bzr)
+    seedtexts = {}
     for seedname in seednames:
-        g.plantSeed(Germinate.seeds.open_seed(SEEDS + RELEASE, seedname, bzr),
+        seed_fd = Germinate.seeds.open_seed(SEEDS, seedbranches, seedname, bzr)
+        seedtexts[seedname] = seed_fd.readlines()
+        seed_fd.close()
+        g.plantSeed(seedtexts[seedname],
                     ARCH, seedname, list(seedinherit[seedname]), RELEASE)
     for seed_package in seed_packages:
         (parent, pkg) = seed_package.split('/')
@@ -414,7 +430,8 @@ def main():
         write_list(seedname, seedname + ".build-depends",
                    g, g.build_depends[seedname])
 
-        if seedname != "extra":
+        if seedname != "extra" and seedname in seedtexts:
+            write_seedtext(seedname + ".seedtext", seedtexts[seedname])
             write_source_list(seedname + ".sources",
                               g, g.sourcepkgs[seedname])
         write_source_list(seedname + ".build-sources",
@@ -432,7 +449,7 @@ def main():
         all_srcs.update(g.sourcepkgs[seedname])
         all_srcs.update(g.build_sourcepkgs[seedname])
 
-        if seedname == "supported":
+        if seedname == g.supported:
             sup.update(g.seed[seedname])
             sup.update(g.seedrecommends[seedname])
             sup.update(g.depends[seedname])
@@ -444,7 +461,7 @@ def main():
         # output.
         build_depends = dict.fromkeys(g.build_depends[seedname], True)
         build_sourcepkgs = dict.fromkeys(g.build_sourcepkgs[seedname], True)
-        for seed in g.innerSeeds("supported"):
+        for seed in g.innerSeeds(g.supported):
             build_depends.update(dict.fromkeys(g.seed[seed], False))
             build_depends.update(dict.fromkeys(g.seedrecommends[seed], False))
             build_depends.update(dict.fromkeys(g.depends[seed], False))
@@ -455,8 +472,8 @@ def main():
     write_list("all", "all", g, all)
     write_source_list("all.sources", g, all_srcs)
 
-    write_list("all", "supported+build-depends", g, sup)
-    write_source_list("supported+build-depends.sources", g, sup_srcs)
+    write_list("all", "%s+build-depends" % g.supported, g, sup)
+    write_source_list("%s+build-depends.sources" % g.supported, g, sup_srcs)
 
     write_list("all", "all+extra", g, g.all)
     write_source_list("all+extra.sources", g, g.all_srcs)
