@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 
-# Copyright (c) 2004, 2005, 2006, 2007 Canonical Ltd.
+# Copyright (c) 2004, 2005, 2006, 2007, 2008 Canonical Ltd.
 #
 # This file is part of Germinate.
 #
@@ -33,10 +33,10 @@ import Germinate.seeds
 import Germinate.version
 
 # TODO: cloned from germinate.py; should be common
-SEEDS = "http://people.ubuntu.com/~ubuntu-archive/seeds/"
-RELEASE = "ubuntu.gutsy"
-MIRROR = "http://archive.ubuntu.com/ubuntu/"
-DIST = ["gutsy"]
+SEEDS = ["http://people.ubuntu.com/~ubuntu-archive/seeds/"]
+RELEASE = "ubuntu.hardy"
+MIRRORS = ["http://archive.ubuntu.com/ubuntu/"]
+DIST = ["hardy"]
 COMPONENTS = ["main"]
 ARCH = "i386"
 
@@ -94,25 +94,32 @@ class Globals:
         # Suppress most log information
         logging.getLogger().setLevel(logging.CRITICAL)
 
-        global RELEASE, MIRROR, DIST, COMPONENTS, ARCH
+        global RELEASE, MIRRORS, DIST, COMPONENTS, ARCH
         print "Germinating"
         g = Germinator()
         apt_pkg.InitConfig()
         apt_pkg.Config.Set("APT::Architecture", ARCH)
+        apt_pkg.InitSystem()
 
-        Germinate.Archive.TagFile(MIRROR).feed(g, DIST, COMPONENTS, ARCH, True)
+        Germinate.Archive.TagFile(MIRRORS).feed(
+            g, DIST, COMPONENTS, ARCH, True)
 
-        (seednames, seedinherit) = g.parseStructure(
-            Germinate.seeds.open_seed(SEEDS + RELEASE, "STRUCTURE"))
+        seednames, seedinherit, seedbranches = g.parseStructure(SEEDS, RELEASE)
         needed_seeds = []
+        build_tree = False
         for seedname in self.seeds:
+            if seedname == ('%s+build-depends' % g.supported):
+                seedname = g.supported
+                build_tree = True
             for inherit in seedinherit[seedname]:
                 if inherit not in needed_seeds:
                     needed_seeds.append(inherit)
-            needed_seeds.append(seedname)
+            if seedname not in needed_seeds:
+                needed_seeds.append(seedname)
         for seedname in needed_seeds:
-            g.plantSeed(Germinate.seeds.open_seed(SEEDS + RELEASE, seedname),
-                        ARCH, seedname, list(seedinherit[seedname]), RELEASE)
+            g.plantSeed(
+                Germinate.seeds.open_seed(SEEDS, seedbranches, seedname),
+                ARCH, seedname, list(seedinherit[seedname]), RELEASE)
         g.prune()
         g.grow()
 
@@ -127,6 +134,19 @@ class Globals:
                 self.package.setdefault(pkg, Package(pkg))
                 self.package[pkg].setSeed(seedname + ".depends")
 
+            if build_tree:
+                build_depends = dict.fromkeys(g.build_depends[seedname], True)
+                for inner in g.innerSeeds(g.supported):
+                    build_depends.update(dict.fromkeys(g.seed[inner], False))
+                    build_depends.update(dict.fromkeys(g.seedrecommends[inner],
+                                                       False))
+                    build_depends.update(dict.fromkeys(g.depends[inner],
+                                                       False))
+                for (pkg, use) in build_depends.iteritems():
+                    if use:
+                        self.package.setdefault(pkg, Package(pkg))
+                        self.package[pkg].setSeed(g.supported + ".build-depends")
+
     def parseDpkg(self, fname):
         if fname == None:
             f = os.popen("dpkg --get-selections")
@@ -161,23 +181,31 @@ Options:
   -l, --list=FILE       Read list of packages from this file
                         (default: read from dpkg --get-selections).
   -m, --mode=[i|r|d]    Show packages to install/remove/diff (default: d).
+  -S, --seed-source=SOURCE
+                        Fetch seeds from SOURCE
+                        (default: %s).
+  -s, --seed-dist=DIST  Fetch seeds for distribution DIST (default: %s).
+  -d, --dist=DIST       Operate on distribution DIST (default: %s).
   -a, --arch=ARCH       Operate on architecture ARCH (default: %s).
 
 A list of seeds against which to compare may be supplied as non-option
 arguments. Seeds from which they inherit will be added automatically. The
 default is 'desktop'.
-""" % ARCH
+""" % (",".join(SEEDS), RELEASE, ",".join(DIST), ARCH)
 
 def main():
-    global ARCH
+    global SEEDS, RELEASE, DIST, ARCH
     g = Globals()
 
     try:
-        opts, args = getopt.getopt(sys.argv[1:], "hl:m:a:",
+        opts, args = getopt.getopt(sys.argv[1:], "hl:m:S:s:d:a:",
                                    ["help",
                                     "version",
                                     "list=",
                                     "mode=",
+                                    "seed-source=",
+                                    "seed-dist=",
+                                    "dist=",
                                     "arch="])
     except getopt.GetoptError:
         usage(sys.stderr)
@@ -197,6 +225,12 @@ def main():
         elif option in ("-m", "--mode"):
             # one of 'i' (install), 'r' (remove), or 'd' (default)
             g.setOutput(value)
+        elif option in ("-S", "--seed-source"):
+            SEEDS = value.split(",")
+        elif option in ("-s", "--seed-dist"):
+            RELEASE = value
+        elif option in ("-d", "--dist"):
+            DIST = value.split(",")
         elif option in ("-a", "--arch"):
             ARCH = value
 
