@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 
-# Copyright (c) 2004, 2005, 2006, 2007 Canonical Ltd.
+# Copyright (c) 2004, 2005, 2006, 2007, 2008 Canonical Ltd.
 # Copyright (c) 2006 Gustavo Franco
 #
 # This file is part of Germinate.
@@ -109,6 +109,7 @@ except ConfigParser.NoOptionError:
 architectures = config.get(dist, 'architectures').split()
 try:
     archive_base_default = config.get(dist, 'archive_base/default')
+    archive_base_default = re.split(r'[, ]+', archive_base_default)
 except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
     archive_base_default = None
 
@@ -116,6 +117,7 @@ archive_base = {}
 for arch in architectures:
     try:
         archive_base[arch] = config.get(dist, 'archive_base/%s' % arch)
+        archive_base[arch] = re.split(r'[, ]+', archive_base[arch])
     except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
         if archive_base_default is not None:
             archive_base[arch] = archive_base_default
@@ -126,31 +128,46 @@ if bzr and config.has_option("%s/bzr" % dist, 'seed_base'):
     seed_base = config.get("%s/bzr" % dist, 'seed_base')
 else:
     seed_base = config.get(dist, 'seed_base')
-if not seed_base.endswith('/'):
-    seed_base += '/'
+seed_base = re.split(r'[, ]+', seed_base)
 if bzr and config.has_option("%s/bzr" % dist, 'seed_dist'):
     seed_dist = config.get("%s/bzr" % dist, 'seed_dist')
 elif config.has_option(dist, 'seed_dist'):
     seed_dist = config.get(dist, 'seed_dist')
 else:
     seed_dist = dist
-seed_base += seed_dist
 seed_entry = re.compile(' *\* *(?P<package>\S+) *(\[(?P<arches>[^]]*)\])? *(#.*)?')
 components = config.get(dist, 'components').split()
 
-def seed_packages(germinator_list, seed):
+def seed_packages(germinator_list, seed, seed_text):
     if config.has_option(dist, "seed_map/%s" % seed):
         mapped_seeds = config.get(dist, "seed_map/%s" % seed).split()
     else:
-        mapped_seeds = [seed]
+        mapped_seeds = []
+        task_seeds_re = re.compile('^Task-Seeds:\s*(.*)', re.I)
+        for line in seed_text:
+            task_seeds_match = task_seeds_re.match(line)
+            if task_seeds_match is not None:
+                mapped_seeds = task_seeds_match.group(1).split()
+                break
+        if seed not in mapped_seeds:
+            mapped_seeds.append(seed)
     packages = []
     for mapped_seed in mapped_seeds:
         packages.extend(germinator_list[mapped_seed])
     return packages
 
+def metapackage_name(seed, seed_text):
+    if config.has_option(dist, "metapackage_map/%s" % seed):
+        return config.get(dist, "metapackage_map/%s" % seed)
+    else:
+        task_meta_re = re.compile('^Task-Metapackage:\s*(.*)', re.I)
+        for line in seed_text:
+            task_meta_match = task_meta_re.match(line)
+            if task_meta_match is not None:
+                return task_meta_match.group(1)
+        return "%s-%s" % (metapackage, seed)
+
 debootstrap_version_file = 'debootstrap-version'
-metapackages = map(lambda seed: '%s-%s' % (metapackage, seed), seeds)
-seed_package_blacklist = set(metapackages)
 
 def get_debootstrap_version():
     version = os.popen("dpkg-query -W --showformat '${Version}' debootstrap").read()
@@ -160,7 +177,7 @@ def get_debootstrap_version():
     return version
 
 def debootstrap_packages(arch):
-    debootstrap = os.popen('debootstrap --arch %s --print-debs %s debootstrap-dir %s' % (arch,dist,archive_base[arch]))
+    debootstrap = os.popen('debootstrap --arch %s --print-debs %s debootstrap-dir %s' % (arch,dist,archive_base[arch][0]))
     packages = debootstrap.read().split()
     if debootstrap.close():
         raise RuntimeError('Unable to retrieve package list from debootstrap')
@@ -198,7 +215,9 @@ check_debootstrap_version()
 
 additions = {}
 removals = {}
+moves = {}
 apt_pkg.InitConfig()
+apt_pkg.InitSystem()
 for architecture in architectures:
     print "[%s] Downloading available package lists..." % architecture
     apt_pkg.Config.Set("APT::Architecture", architecture)
@@ -208,16 +227,28 @@ for architecture in architectures:
     debootstrap_base = set(debootstrap_packages(architecture))
 
     print "[%s] Loading seed lists..." % architecture
-    (seed_names, seed_inherit) = germinator.parseStructure(
-        Germinate.seeds.open_seed(seed_base, "STRUCTURE", bzr))
+    seed_names, seed_inherit, seed_branches = germinator.parseStructure(
+        seed_base, seed_dist, bzr)
+    needed_seeds = []
     for seed_name in seeds:
-        germinator.plantSeed(Germinate.seeds.open_seed(seed_base, seed_name,
-                                                       bzr),
-                             architecture, seed_name,
+        for inherit in seed_inherit[seed_name]:
+            if inherit not in needed_seeds:
+                needed_seeds.append(inherit)
+        if seed_name not in needed_seeds:
+            needed_seeds.append(seed_name)
+    seed_texts = {}
+    for seed_name in needed_seeds:
+        seed_fd = Germinate.seeds.open_seed(seed_base, seed_branches,
+                                            seed_name, bzr)
+        seed_texts[seed_name] = seed_fd.readlines()
+        seed_fd.close()
+        germinator.plantSeed(seed_texts[seed_name], architecture, seed_name,
                              list(seed_inherit[seed_name]))
 
     print "[%s] Merging seeds with available package lists..." % architecture
     for seed_name in output_seeds:
+        meta_name = metapackage_name(seed_name, seed_texts[seed_name])
+
         output_filename = '%s-%s' % (seed_name,architecture)
         old_list = None
         if os.path.exists(output_filename):
@@ -226,8 +257,10 @@ for architecture in architectures:
 
         # work on the depends
         new_list = []
-        for package in seed_packages(germinator.seed, seed_name):
-            if package in seed_package_blacklist:
+        for package in seed_packages(germinator.seed, seed_name,
+                                     seed_texts[seed_name]):
+            if package == meta_name:
+                print "%s/%s: Skipping package %s (metapackage)" % (seed_name,architecture,package)
                 continue
             if seed_name == 'minimal' and package not in debootstrap_base:
                 print "%s/%s: Skipping package %s (package not in debootstrap)" % (seed_name,architecture,package)
@@ -244,8 +277,10 @@ for architecture in architectures:
         # work on the recommends
         old_recommends_list = None
         new_recommends_list = []
-        for package in seed_packages(germinator.seedrecommends, seed_name):
-            if package in seed_package_blacklist:
+        for package in seed_packages(germinator.seedrecommends, seed_name,
+                                     seed_texts[seed_name]):
+            if package == meta_name:
+                print "%s/%s: Skipping package %s (metapackage)" % (seed_name,architecture,package)
                 continue
             if seed_name == 'minimal' and package not in debootstrap_base:
                 print "%s/%s: Skipping package %s (package not in debootstrap)" % (seed_name,architecture,package)
@@ -266,29 +301,16 @@ for architecture in architectures:
 
 
         # Calculate deltas
+        merged = {}
+        recommends_merged = {}
         if old_list is not None:
-            merged = {}
             for package in new_list:
                 merged.setdefault(package, 0)
                 merged[package] += 1
             for package in old_list:
                 merged.setdefault(package, 0)
                 merged[package] -= 1
-
-            mergeditems = merged.items()
-            mergeditems.sort()
-            for package, value in mergeditems:
-                #print package, value
-                if value == 1:
-                    additions.setdefault(package,[])
-                    additions[package].append(output_filename)
-                elif value == -1:
-                    removals.setdefault(package,[])
-                    removals[package].append(output_filename)
-
-        # now the recommends
         if old_recommends_list is not None:
-            recommends_merged = {}
             for package in new_recommends_list:
                 recommends_merged.setdefault(package, 0)
                 recommends_merged[package] += 1
@@ -296,16 +318,37 @@ for architecture in architectures:
                 recommends_merged.setdefault(package, 0)
                 recommends_merged[package] -= 1
 
-            mergedrecitems = recommends_merged.items()
-            mergedrecitems.sort()
-            for package, value in mergedrecitems:
-                #print package, value
-                if value == 1:
+        mergeditems = merged.items()
+        mergeditems.sort()
+        for package, value in mergeditems:
+            #print package, value
+            if value == 1:
+                if recommends_merged.get(package, 0) == -1:
+                    moves.setdefault(package,[])
+                    moves[package].append(output_filename)
+                    recommends_merged[package] += 1
+                else:
                     additions.setdefault(package,[])
-                    additions[package].append(output_recommends_filename)
-                elif value == -1:
+                    additions[package].append(output_filename)
+            elif value == -1:
+                if recommends_merged.get(package, 0) == 1:
+                    moves.setdefault(package,[])
+                    moves[package].append(output_recommends_filename)
+                    recommends_merged[package] -= 1
+                else:
                     removals.setdefault(package,[])
-                    removals[package].append(output_recommends_filename)
+                    removals[package].append(output_filename)
+
+        mergedrecitems = recommends_merged.items()
+        mergedrecitems.sort()
+        for package, value in mergedrecitems:
+            #print package, value
+            if value == 1:
+                additions.setdefault(package,[])
+                additions[package].append(output_recommends_filename)
+            elif value == -1:
+                removals.setdefault(package,[])
+                removals[package].append(output_recommends_filename)
 
 
 if additions or removals:
@@ -327,6 +370,15 @@ if additions or removals:
     for package in removal_keys:
         changes.append('Removed %s from %s' %
                        (package, ', '.join(removals[package])))
+    move_keys = moves.keys()
+    move_keys.sort()
+    for package in move_keys:
+        # TODO: We should really list where it moved from as well, but that
+        # gets wordy very quickly, and at the moment this is only
+        # implemented for depends->recommends or vice versa. In future,
+        # using this for moves between seeds might also be useful.
+        changes.append('Moved %s to %s' %
+                       (package, ', '.join(moves[package])))
     for change in changes:
         print change
         os.system("dch -a '%s'" % change)
